# Copyright 2025 Arseny Seliverstov
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#     http://www.apache.org/licenses/LICENSE-2.0
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Core components
from .application.decorators import provider, singleton, transient
from .application.global_container import init, inject
from .entities.transient import Transient
from .infrastructure.context import context
from .repositories.container_repository import Container
from .value_objects.dependency_marker import DependencyMarker, Depends
from .value_objects.scope import Scope

__all__ = [
    # Core
    "Container",
    "Scope",
    "Transient",
    "Depends",
    "DependencyMarker",
    # Decorators
    "provider",
    "singleton",
    "transient",
    "inject",
    "init",
    # Context
    "context",
]
